@@ -22,6 +22,10 @@ class Geocoder():
                             ...}
     preprocessors   -- list of universal preprocessors to use
     postprocessors  -- list of universal postprocessors to use
+    timeout_secs    -- seconds to wait for a geocoding source before moving
+                       on to the next one (default 5)
+    waterfall       -- sets default for waterfall on geocode() method
+                       (default False)
     """
 
     DEFAULT_SOURCES = [['omgeo.services.EsriNA', {}],
@@ -43,8 +47,8 @@ class Geocoder():
             for p in path:
                 m = getattr(m, p)
             return m
-        except:
-            raise Exception("No geocoder with the name %s" % service_name)
+        except Exception as ex:
+            raise Exception("%s" % (ex))
 
     def add_source(self, source):
         geocode_service = self._get_service_by_name(source[0])
@@ -69,19 +73,16 @@ class Geocoder():
         for source in sources: # iterate through a list of sources
             self.add_source(source)
 
-    def __init__(self, sources=None, preprocessors=None, postprocessors=None):
+    def __init__(self, sources=None, preprocessors=None, postprocessors=None,
+                 waterfall=False):
         self._preprocessors = Geocoder.DEFAULT_PREPROCESSORS \
             if preprocessors is None else preprocessors
-        
         self._postprocessors = Geocoder.DEFAULT_POSTPROCESSORS \
             if postprocessors is None else postprocessors
-
-        # Reserved for future use
-        self._settings = {}
-    
         sources = Geocoder.DEFAULT_SOURCES if sources is None else sources
         self.set_sources(sources)
-
+        self.waterfall = waterfall
+        
     def geocode(self, pq, waterfall=None):
         """
         Returns a list of Candidate objects
@@ -92,18 +93,19 @@ class Geocoder():
         waterfall   --  Boolean set to True if all geocoders listed should
                         be used to find results, instead of stopping after
                         the first geocoding service with valid candidates
-                        (default False).
+                        (defaults to <Geocoder instance>.waterfall).
         """
         start_time = time.time()
-        waterfall = self._settings.get('waterfall', False)
+        waterfall = self.waterfall if waterfall is None else waterfall
         if type(pq) in (str, unicode):
             pq = PlaceQuery(pq)
         processed_pq = copy.copy(pq)
+        
         for p in self._preprocessors: # apply universal address preprocessing
             processed_pq = p.process(processed_pq)
             if processed_pq == False:
                 return []
-
+            
         processed_candidates = []
         for gs in self._sources: # iterate through each GeocodeService
             logger.debug('%s: Geocoding using %s...' % ((time.time() - start_time), gs))
@@ -117,4 +119,5 @@ class Geocoder():
                 return [] # avoid post-processing empty list
             logger.debug('%s: Applying universal postprocessor %s...' % ((time.time() - start_time), p))
             processed_candidates = p.process(processed_candidates)
+            
         return processed_candidates
