@@ -587,6 +587,9 @@ class MapQuest(GeocodeService):
             c.geoservice = self.__class__.__name__
             returned_candidates.append(c)
         return returned_candidates
+    
+class MapQuestSSL(MapQuest):
+    _endpoint = 'https://www.mapquestapi.com/geocoding/v1/address'
 
 class Nominatim(GeocodeService):
     """
