@@ -6,8 +6,8 @@ import sys
 import unittest
 from omgeo import Geocoder
 from omgeo.places import Viewbox, PlaceQuery, Candidate
-from omgeo.processors.preprocessors import CountryPreProcessor, RequireCountry, ParseSingleLine,\
-                                           ReplaceRangeWithNumber
+from omgeo.processors.preprocessors import CancelIfRegexInAttr, CountryPreProcessor, RequireCountry,\
+                                           ParseSingleLine, ReplaceRangeWithNumber
 from omgeo.processors.postprocessors import AttrFilter, AttrExclude, AttrRename,\
                                             AttrSorter, AttrReverseSorter, UseHighScoreIfAtLeast,\
                                             GroupBy, GroupByMultiple, ScoreSorter, SnapPoints
@@ -17,6 +17,9 @@ ESRI_MAPS_API_KEY = os.getenv("ESRI_MAPS_API_KEY")
 MAPQUEST_API_KEY = os.getenv("MAPQUEST_API_KEY")
 
 logger = logging.getLogger(__name__)
+logging.basicConfig(level='INFO')
+#logging.basicConfig(level='DEBUG')
+
 
 class OmgeoTestCase(unittest.TestCase):
     def assertEqual_(self, output, expected):
@@ -84,6 +87,8 @@ class GeocoderTest(OmgeoTestCase):
             mapquest_settings = dict(api_key=MAPQUEST_API_KEY)
             self.service_mapquest = ['omgeo.services.MapQuest', dict(settings=mapquest_settings)]
             self.g_mapquest = Geocoder([self.service_mapquest])
+            self.service_mapquest_ssl = ['omgeo.services.MapQuestSSL', dict(settings=mapquest_settings)]
+            self.g_mapquest_ssl = Geocoder([self.service_mapquest_ssl])            
         self.service_nom = ['omgeo.services.Nominatim', {}]
         g_sources.append(self.service_nom)
         self.g_nom = Geocoder([self.service_nom])
@@ -92,6 +97,7 @@ class GeocoderTest(OmgeoTestCase):
         self.g_dc = Geocoder([['omgeo.services.CitizenAtlas', {}]])
         self.g_esri_na_soap = Geocoder([['omgeo.services.EsriNASoap', {}]])
         self.g_esri_eu_soap = Geocoder([['omgeo.services.EsriEUSoap', {}]])
+        self.impatient_geocoder = Geocoder([['omgeo.services.EsriNA', dict(settings=dict(timeout=0.01))]])
         
     def tearDown(self):
         pass
@@ -101,6 +107,10 @@ class GeocoderTest(OmgeoTestCase):
         self.assertEqual(len(candidates) > 0, True, 'No candidates returned.')
         self.assertEqual(len(candidates) > 1, False, 'More than one candidate returned.')
         
+    def test_impatiently_geocode_azavea(self):
+        candidates = self.impatient_geocoder.geocode(self.pq['azavea'])
+        self.assertEqual(len(candidates) == 0, True, 'Candidates were unexpectedly returned in under 10ms.')
+        
     def test_geocode_snap_points_1(self):
         candidates = self.g.geocode(self.pq['8_kirkbride'])
         self.assertEqual(len(candidates) > 0, True, 'No candidates returned.')
@@ -140,6 +150,12 @@ class GeocoderTest(OmgeoTestCase):
     def test_geocode_mapquest(self):
         candidates = self.g_mapquest.geocode(self.pq['azavea'])
         self.assertEqual(len(candidates) > 0, True, 'No candidates returned.')
+
+    @unittest.skipIf(MAPQUEST_API_KEY is None, MAPQUEST_KEY_REQUIRED_MSG)
+    def test_geocode_mapquest(self):
+        candidates = self.g_mapquest_ssl.geocode(self.pq['azavea'])
+        self.assertEqual(len(candidates) > 0, True, 'No candidates returned.')
+        
         
     def test_geocode_nom(self):
         candidates = self.g_nom.geocode(PlaceQuery('1200 Callowhill St, Philadelphia, PA, 19123'))
@@ -286,6 +302,18 @@ class GeocoderProcessorTest(OmgeoTestCase):
         place_out = RequireCountry().process(place_in)
         place_exp = False
         self.assertEqual_(place_out, place_exp)
+        
+    def test_pro_CancelIfRegexInAttr(self):
+        place_in = PlaceQuery('PO Box 123, Philadelphia, PA')
+        place_out = CancelIfRegexInAttr(regex="po box", attrs=('query',)).process(place_in)
+        place_exp = False
+        self.assertEqual_(place_out, place_exp)
+        
+    def test_pro_CancelIfRegexInAttr_case_sensitive(self):
+        place_in = PlaceQuery('PO Box 123, Philadelphia, PA')
+        place_out = CancelIfRegexInAttr(regex="PO BOX", attrs=('query',), ignorecase=False).process(place_in)
+        place_exp = place_in # we should still have it because PO BOX does not match exactly
+        self.assertEqual_(place_out, place_exp)
 
     def test_pro_filter_AttrFilter_exact(self):
         good_values = ['roof', 'parcel']
